function [A,pivs,tol] = ref(A,tol)
% ref  
%   R = ref(A) produces the reduced row echelon form of A.
%   [R pivcols] = ref(A) also produces a vector listing the pivot columns.
%   Using ref(A,tol) allows user to specify tolerance for rank tests. 
%
%       This is Cleve Moler's funtion RREF, with the rational tests   
% removed so it runs more quickly.  
%       It does partial pivoting like the lu function does.  However, ref   
% zeros out above and below each pivot entry in the "forward sweep" step, 
% whereas lu does the more efficient "forward sweep, back sweep" method. 
%                               J. Day, 12/96
[m,n] = size(A);

% Compute the default tolerance if none was provided: 
if (nargin < 2), tol = max(m,n)*eps*norm(A,'inf'); end
%eps,tol
% Loop over the entire matrix:
pivs = [];
i = 1;
j = 1;
while (i <= m) & (j <= n)
   % Find value & index of largest entry in column j below pivot position.
   [p,k] = max(abs(A(i:m,j))); k = k+i-1;
   if (p <= tol)
      % The column is negligible, zero it out.
      A(i:m,j) = zeros(m-i+1,1);
      j = j + 1; 
   else
      pivs = [pivs j];
      % Swap i-th and k-th rows.
      A([i k],j:n) = A([k i],j:n);
      % Divide the pivot row by the pivot element.
      A(i,j:n) = A(i,j:n)/A(i,j);
      % Subtract multiples of the pivot row from all the other rows.
      for k = [1:i-1 i+1:m]
         A(k,j:n) = A(k,j:n) - A(k,j)*A(i,j:n);
      end
      i = i + 1;
      j = j + 1;
   end
end
